@@ -1,4 +1,4 @@
-# Copyright 2021 The TensorFlow Recommenders Authors.
+# Copyright 2022 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lint as: python3
 """Functions supporting Movielens examples."""
 
 import array
@@ -72,7 +71,7 @@ def evaluate(user_model: tf.keras.Model,
   precision_values = []
   recall_values = []
 
-  for (user_id, test_movies) in test_user_to_movies.items():
+  for user_id, test_movies in test_user_to_movies.items():
     user_embedding = user_model({"user_id": np.array([user_id])}).numpy()
     scores = (user_embedding @ movie_embeddings.T).flatten()
 
@@ -167,10 +166,9 @@ def sample_listwise(
   for example in rating_dataset:
     user_id = example["user_id"].numpy()
     example_lists_by_user[user_id]["movie_title"].append(
-        example["movie_title"],)
+        example["movie_title"])
     example_lists_by_user[user_id]["user_rating"].append(
-        example["user_rating"],
-    )
+        example["user_rating"])
     movie_title_vocab.add(example["movie_title"].numpy())
 
   tensor_slices = {"user_id": [], "movie_title": [], "user_rating": []}
