@@ -1,4 +1,4 @@
-# Copyright 2021 The TensorFlow Recommenders Authors.
+# Copyright 2022 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -48,19 +48,26 @@ class CompositeOptimizerTest(tf.test.TestCase, parameterized.TestCase):
     grads2 = tf.constant(grad2_values)
     grads3 = tf.constant(grad3_values)
 
-    comp_optimizer1 = tf.keras.optimizers.get(optimizer1_type)
-    comp_optimizer2 = tf.keras.optimizers.get(optimizer2_type)
+    optimizer_dict = {
+        "sgd": tf.keras.optimizers.legacy.SGD,
+        "adam": tf.keras.optimizers.legacy.Adam,
+        "rmsprop": tf.keras.optimizers.legacy.RMSprop,
+        "adagrad": tf.keras.optimizers.legacy.Adagrad,
+    }
+
+    comp_optimizer1 = optimizer_dict[optimizer1_type]()
+    comp_optimizer2 = optimizer_dict[optimizer2_type]()
 
     composite_optimizer = CompositeOptimizer([
         (comp_optimizer1, lambda: [var1]),
         (comp_optimizer2, lambda: [var2, var3]),
     ])
 
-    self.assertSequenceEqual(
-        composite_optimizer.optimizers, [comp_optimizer1, comp_optimizer2])
+    self.assertSequenceEqual(composite_optimizer.optimizers,
+                             [comp_optimizer1, comp_optimizer2])
 
-    optimizer1 = tf.keras.optimizers.get(optimizer1_type)
-    optimizer2 = tf.keras.optimizers.get(optimizer2_type)
+    optimizer1 = optimizer_dict[optimizer1_type]()
+    optimizer2 = optimizer_dict[optimizer2_type]()
 
     grads_and_vars_1 = [(tf.constant(grad1_values), tf.Variable(values1))]
     grads_and_vars_2 = [(tf.constant(grad2_values), tf.Variable(values2)),
@@ -89,8 +96,8 @@ class CompositeOptimizerTest(tf.test.TestCase, parameterized.TestCase):
 
     # Test same variable in two optimizers.
     composite_optimizer = CompositeOptimizer([
-        (tf.keras.optimizers.Adam(), lambda: [var1]),
-        (tf.keras.optimizers.Adagrad(), lambda: [var1, var2]),
+        (tf.keras.optimizers.legacy.Adam(), lambda: [var1]),
+        (tf.keras.optimizers.legacy.Adagrad(), lambda: [var1, var2]),
     ])
 
     grads_and_vars = list(zip([grads1, grads2], [var1, var2]))
@@ -100,8 +107,8 @@ class CompositeOptimizerTest(tf.test.TestCase, parameterized.TestCase):
 
     # Test missing variable (var3) in optimizers.
     composite_optimizer = CompositeOptimizer([
-        (tf.keras.optimizers.Adam(), lambda: [var1]),
-        (tf.keras.optimizers.Adagrad(), lambda: [var2]),
+        (tf.keras.optimizers.legacy.Adam(), lambda: [var1]),
+        (tf.keras.optimizers.legacy.Adagrad(), lambda: [var2]),
     ])
 
     grads_and_vars = list(zip([grads1, grads2, grads3], [var1, var2, var3]))
@@ -110,14 +117,14 @@ class CompositeOptimizerTest(tf.test.TestCase, parameterized.TestCase):
       composite_optimizer.apply_gradients(grads_and_vars)
 
   def test_checkpoint_save_restore_export(self):
-    # Use a simple Linear model to test checkpoint save/restore/export.
+    # Use a simple LinearModel to test checkpoint save/restore/export.
     def get_model() -> tf.keras.Model:
       model = tf.keras.experimental.LinearModel(units=10)
 
       composite_optimizer = CompositeOptimizer([
-          (tf.keras.optimizers.Adam(),
+          (tf.keras.optimizers.legacy.Adam(),
            lambda: model.trainable_variables[:1]),
-          (tf.keras.optimizers.Adagrad(),
+          (tf.keras.optimizers.legacy.Adagrad(),
            lambda: model.trainable_variables[1:]),
       ])
       model.compile(optimizer=composite_optimizer,
