@@ -1,4 +1,4 @@
-# Copyright 2021 The TensorFlow Recommenders Authors.
+# Copyright 2022 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -98,5 +98,34 @@ class LossTest(tf.test.TestCase, parameterized.TestCase):
                                                                    col_idx])
 
 
+class SamplingProbabilityCorrectionTest(
+    tf.test.TestCase, parameterized.TestCase):
+  """Loss layers tests."""
+
+  @parameterized.parameters(42, 123, 8391, 12390, 1230)
+  def test_sampling_probability_correction(self, random_seed):
+    """Test sampling probability correction."""
+
+    shape = (10, 10)
+    rng = np.random.RandomState(random_seed)
+
+    logits = rng.uniform(size=shape).astype(np.float32)
+    probs = rng.uniform(size=shape[0]).astype(np.float32)
+
+    corrected_logits = loss.SamplingProbablityCorrection()(logits, probs)
+    corrected_logits = corrected_logits.numpy()
+
+    np.testing.assert_array_less(logits, corrected_logits)
+
+    # set some of the probabilities to 0
+    probs_with_zeros = probs * rng.choice([0., 1.], size=probs.shape)
+
+    corrected_logits_with_zeros = loss.SamplingProbablityCorrection()(
+        logits, probs_with_zeros)
+    corrected_logits_with_zeros = corrected_logits_with_zeros.numpy()
+
+    np.testing.assert_array_less(logits, corrected_logits_with_zeros)
+
+
 if __name__ == "__main__":
   tf.test.main()
