@@ -1,4 +1,4 @@
-# Copyright 2021 The TensorFlow Recommenders Authors.
+# Copyright 2022 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,8 +43,10 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
                loss: Optional[tf.keras.losses.Loss] = None,
                metrics: Optional[tfrs_metrics.FactorizedTopK] = None,
                batch_metrics: Optional[List[tf.keras.metrics.Metric]] = None,
+               loss_metrics: Optional[List[tf.keras.metrics.Metric]] = None,
                temperature: Optional[float] = None,
                num_hard_negatives: Optional[int] = None,
+               remove_accidental_hits: bool = False,
                name: Optional[Text] = None) -> None:
     """Initializes the task.
 
@@ -61,11 +63,16 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
        true candidate for a query from other candidates in the batch. For
        example, a batch AUC metric would measure the probability that the true
        candidate is scored higher than the other candidates in the batch.
+      loss_metrics: List of Keras metrics used to summarize the loss.
       temperature: Temperature of the softmax.
       num_hard_negatives: If positive, the `num_hard_negatives` negative
         examples with largest logits are kept when computing cross-entropy loss.
         If larger than batch size or non-positive, all the negative examples are
         kept.
+      remove_accidental_hits: When given
+        enables removing accidental hits of examples used as negatives. An
+        accidental hit is defined as a candidate that is used as an in-batch
+        negative but has the same id with the positive candidate.
       name: Optional task name.
     """
 
@@ -75,9 +82,11 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
         from_logits=True, reduction=tf.keras.losses.Reduction.SUM)
 
     self._factorized_metrics = metrics
-    self._batch_metrics = batch_metrics
+    self._batch_metrics = batch_metrics or []
+    self._loss_metrics = loss_metrics or []
     self._temperature = temperature
     self._num_hard_negatives = num_hard_negatives
+    self._remove_accidental_hits = remove_accidental_hits
 
   @property
   def factorized_metrics(self) -> Optional[tfrs_metrics.FactorizedTopK]:
@@ -98,7 +107,8 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
            sample_weight: Optional[tf.Tensor] = None,
            candidate_sampling_probability: Optional[tf.Tensor] = None,
            candidate_ids: Optional[tf.Tensor] = None,
-           compute_metrics: bool = True) -> tf.Tensor:
+           compute_metrics: bool = True,
+           compute_batch_metrics: bool = True) -> tf.Tensor:
     """Computes the task loss and metrics.
 
     The main argument are pairs of query and candidate embeddings: the first row
@@ -118,13 +128,12 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
       candidate_sampling_probability: Optional tensor of candidate sampling
         probabilities. When given will be be used to correct the logits to
         reflect the sampling probability of negative candidates.
-      candidate_ids: Optional tensor containing candidate ids. When given
-        enables removing accidental hits of examples used as negatives. An
-        accidental hit is defined as an candidate that is used as an in-batch
-        negative but has the same id with the positive candidate.
+      candidate_ids: Optional tensor containing candidate ids. When given,
+        factorized top-K evaluation will be id-based rather than score-based.
       compute_metrics: Whether to compute metrics. Set this to False
         during training for faster training.
-
+      compute_batch_metrics: Whether to compute batch level metrics.
+        In-batch loss_metrics will still be computed.
     Returns:
       loss: Tensor of loss values.
     """
@@ -137,18 +146,6 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
 
     labels = tf.eye(num_queries, num_candidates)
 
-    metric_update_ops = []
-    if compute_metrics:
-      if self._factorized_metrics:
-        metric_update_ops.append(
-            self._factorized_metrics.update_state(query_embeddings,
-                                                  candidate_embeddings))
-      if self._batch_metrics:
-        metric_update_ops.extend([
-            batch_metric.update_state(labels, scores)
-            for batch_metric in self._batch_metrics
-        ])
-
     if self._temperature is not None:
       scores = scores / self._temperature
 
@@ -156,7 +153,12 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
       scores = layers.loss.SamplingProbablityCorrection()(
           scores, candidate_sampling_probability)
 
-    if candidate_ids is not None:
+    if self._remove_accidental_hits:
+      if candidate_ids is None:
+        raise ValueError(
+            "When accidental hit removal is enabled, candidate ids "
+            "must be supplied."
+        )
       scores = layers.loss.RemoveAccidentalHits()(labels, scores, candidate_ids)
 
     if self._num_hard_negatives is not None:
@@ -166,10 +168,25 @@ class Retrieval(tf.keras.layers.Layer, base.Task):
 
     loss = self._loss(y_true=labels, y_pred=scores, sample_weight=sample_weight)
 
-    if not metric_update_ops:
-      return loss
+    update_ops = []
+    for metric in self._loss_metrics:
+      update_ops.append(
+          metric.update_state(loss, sample_weight=sample_weight))
 
-    with tf.control_dependencies(metric_update_ops):
+    if self._factorized_metrics is not None and compute_metrics:
+      update_ops.append(
+          self._factorized_metrics.update_state(
+              query_embeddings,
+              # Slice to the size of query embeddings if `candidate_embeddings`
+              # contains extra negatives.
+              candidate_embeddings[:tf.shape(query_embeddings)[0]],
+              true_candidate_ids=candidate_ids)
+      )
+    if compute_batch_metrics:
+      for metric in self._batch_metrics:
+        update_ops.append(metric.update_state(labels, scores))
+
+    with tf.control_dependencies(update_ops):
       return tf.identity(loss)
 
 
