@@ -1,4 +1,4 @@
-# Copyright 2021 The TensorFlow Recommenders Authors.
+# Copyright 2022 The TensorFlow Recommenders Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,10 +38,8 @@ class RetrievalTest(tf.test.TestCase):
     task = retrieval.Retrieval(
         metrics=metrics.FactorizedTopK(
             candidates=candidate_dataset.batch(16),
-            metrics=[
-                tf.keras.metrics.TopKCategoricalAccuracy(
-                    k=5, name="factorized_categorical_accuracy_at_5")
-            ]),
+            ks=[5]
+        ),
         batch_metrics=[
             tf.keras.metrics.TopKCategoricalAccuracy(
                 k=1, name="batch_categorical_accuracy_at_1")
@@ -50,11 +48,11 @@ class RetrievalTest(tf.test.TestCase):
     # All_pair_scores: [[6, 3], [9, 5]].
     # Normalized logits: [[3, 0], [4, 0]].
     expected_loss = -np.log(_sigmoid(3.0)) - np.log(1 - _sigmoid(4.0))
+
     expected_metrics = {
-        "factorized_categorical_accuracy_at_5": 1.0,
+        "factorized_top_k/top_5_categorical_accuracy": 1.0,
         "batch_categorical_accuracy_at_1": 0.5,
     }
-
     loss = task(query_embeddings=query, candidate_embeddings=candidate)
     metrics_ = {
         metric.name: metric.result().numpy() for metric in task.metrics
@@ -64,6 +62,43 @@ class RetrievalTest(tf.test.TestCase):
     self.assertAllClose(expected_loss, loss)
     self.assertAllClose(expected_metrics, metrics_)
 
+    # Test computation of batch metrics when skipping corpus metrics
+    for metric in task.metrics:
+      metric.reset_states()
+    loss = task(query_embeddings=query,
+                candidate_embeddings=candidate,
+                compute_metrics=False)
+    expected_metrics1 = {
+        "factorized_top_k/top_5_categorical_accuracy": 0.0,
+        "batch_categorical_accuracy_at_1": 0.5
+    }
+    metrics1_ = {
+        metric.name: metric.result().numpy() for metric in task.metrics
+    }
+
+    self.assertIsNotNone(loss)
+    self.assertAllClose(expected_loss, loss)
+    self.assertAllClose(expected_metrics1, metrics1_)
+
+    # Test computation of corpus metrics when skipping batch metrics
+    for metric in task.metrics:
+      metric.reset_states()
+    loss = task(
+        query_embeddings=query,
+        candidate_embeddings=candidate,
+        compute_batch_metrics=False)
+    expected_metrics2 = {
+        "factorized_top_k/top_5_categorical_accuracy": 1.0,
+        "batch_categorical_accuracy_at_1": 0.0
+    }
+    metrics2_ = {
+        metric.name: metric.result().numpy() for metric in task.metrics
+    }
+
+    self.assertIsNotNone(loss)
+    self.assertAllClose(expected_loss, loss)
+    self.assertAllClose(expected_metrics2, metrics2_)
+
   def test_task_graph(self):
 
     with tf.Graph().as_default():
@@ -76,17 +111,15 @@ class RetrievalTest(tf.test.TestCase):
         task = retrieval.Retrieval(
             metrics=metrics.FactorizedTopK(
                 candidates=candidate_dataset.batch(16),
-                metrics=[
-                    tf.keras.metrics.TopKCategoricalAccuracy(
-                        k=5, name="factorized_categorical_accuracy_at_5")
-                ]),
+                ks=[5]
+            ),
             batch_metrics=[
                 tf.keras.metrics.TopKCategoricalAccuracy(
                     k=1, name="batch_categorical_accuracy_at_1")
             ])
 
         expected_metrics = {
-            "factorized_categorical_accuracy_at_5": 1.0,
+            "factorized_top_k/top_5_categorical_accuracy": 1.0,
             "batch_categorical_accuracy_at_1": 0.5,
         }
 
